@@ -11,21 +11,27 @@ class DGGBot(DGGChat):
         username: str = None,
         owner: str = None,
         prefix="!",
-        wss: str = None
+        wss: str = None,
     ):
         super().__init__(auth_token=auth_token, username=username, wss=wss)
         self._owner = owner.lower() if owner else None
         self.prefix = prefix
         self._commands = {}
 
-    def command(self, aliases: Union[List[str], Tuple[str]] = tuple(),
-                *args, whisper_only: bool = False,  **kwargs):
+    def command(
+        self,
+        aliases: Union[List[str], Tuple[str]] = tuple(),
+        *args,
+        whisper_only: bool = False,
+        **kwargs,
+    ):
         """
         Decorator to add commands to bot.
-        :param aliases: aliases to call the function besides the function name
+        :param aliases: aliases to call the function besides the function name.
         :param whisper_only: Command will only run when through whispers/private messages.
         :return:
         """
+
         def decorator(func: Callable):
             if whisper_only:
                 func = self.check(lambda msg: isinstance(msg, PrivateMessage))(func)
@@ -56,6 +62,7 @@ class DGGBot(DGGChat):
         :param check_funcs: functions that check for some statement and returns a bool.
         :return:
         """
+
         def decorator(func: Callable):
             if not hasattr(func, "_perms"):
                 func._perms = []
