@@ -1,5 +1,7 @@
 import dataclasses
+from datetime import datetime
 from ._logging import _logger
+from .user import User
 
 
 @dataclasses.dataclass
@@ -7,14 +9,19 @@ class Message:
     chat: "DGGChat"
     type: str
     nick: str = None
+    createdDate: datetime = None
     features: list = None
-    timestamp: int = None
+    timestamp: datetime = None
     data: str = None
 
     @property
     def nick_lower(self) -> str:
         return self.nick.lower()
 
+    @property
+    def user(self) -> User:
+        return User(self.nick, self.createdDate, self.features)
+
     def __post_init__(self):
         _logger.debug(self)
 
